@@ -6,8 +6,8 @@
  * Date        : 2006-09-15
  * Description : Exiv2 library interface for KDE
  *
- * Copyright (C) 2006-2008 by Gilles Caulier <caulier dot gilles at gmail dot com>
- * Copyright (C) 2006-2008 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
+ * Copyright (C) 2006-2009 by Gilles Caulier <caulier dot gilles at gmail dot com>
+ * Copyright (C) 2006-2009 by Marcel Wiesweg <marcel dot wiesweg at gmx dot de>
  *
  * NOTE: Do not use kdDebug() in this implementation because
  *       it will be multithreaded. Use qDebug() instead.
@@ -41,6 +41,7 @@ extern "C"
 #include <qtextcodec.h>
 #include <qwmatrix.h>
 #include <qfileinfo.h>
+#include <qbuffer.h>
 
 // Local includes.
 
@@ -407,10 +408,10 @@ bool KExiv2::save(const QString& filePath)
     // TIFF/EP Raw files based are supported by Exiv2 0.18 as experimental. We will do touch it for the moment.
     // Metadata writing is supported in implementation from svn trunk.
     QString rawTiffBased("dng nef pef 3fr arw cr2 dcr erf k25 kdc mos orf raw sr2 srf"); 
-    if (rawTiffBased.contains(finfo.extension(false).upper()))
+    if (rawTiffBased.contains(finfo.extension(false).lower()))
     {
         qDebug("'%s' is TIFF based RAW file and writing mode is disable with this libkexiv2 version. Metadata not saved.", 
-               dinfo.filePath().ascii());
+               finfo.fileName().ascii());
         return false;
     }
 
@@ -427,8 +428,7 @@ bool KExiv2::save(const QString& filePath)
         // Image Comments ---------------------------------
 
         mode = image->checkMode(Exiv2::mdComment);
-        if (!d->imageComments.empty() && 
-            (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite))
+        if (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite)
         {
             image->setComment(d->imageComments);
         }
@@ -436,8 +436,7 @@ bool KExiv2::save(const QString& filePath)
         // Exif metadata ----------------------------------
 
         mode = image->checkMode(Exiv2::mdExif);
-        if (!d->exifMetadata.empty() && 
-            (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite))
+        if (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite)
         {
             if (image->mimeType() == "image/tiff")
             {
@@ -480,8 +479,7 @@ bool KExiv2::save(const QString& filePath)
         // Iptc metadata ----------------------------------
 
         mode = image->checkMode(Exiv2::mdIptc);
-        if (!d->iptcMetadata.empty() && 
-            (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite))
+        if (mode == Exiv2::amWrite || mode == Exiv2::amReadWrite)
         {
             image->setIptcData(d->iptcMetadata);
         }
@@ -797,15 +795,19 @@ bool KExiv2::setExifThumbnail(const QImage& thumb, bool setProgramName)
 
     try
     {
+#if (EXIV2_TEST_VERSION(0,17,91))
+        QByteArray data;
+        QBuffer buffer(data);
+        buffer.open(IO_WriteOnly);
+        thumb.save(&buffer, "JPEG");
+        Exiv2::ExifThumb thumb(d->exifMetadata);
+        thumb.setJpegThumbnail((Exiv2::byte *)data.data(), data.size());
+#else
         KTempFile thumbFile(QString(), "KExiv2ExifThumbnail");
         thumbFile.setAutoDelete(true);
         thumb.save(thumbFile.name(), "JPEG");
 
         const std::string &fileName( (const char*)(QFile::encodeName(thumbFile.name())) );
-#if (EXIV2_TEST_VERSION(0,17,91))
-        Exiv2::ExifThumb thumb(d->exifMetadata);
-        thumb.setJpegThumbnail( fileName );
-#else
         d->exifMetadata.setJpegThumbnail( fileName );
 #endif
         return true;
@@ -1241,22 +1243,14 @@ bool KExiv2::setImagePreview(const QImage& preview, bool setProgramName)
 
     try
     {
-        KTempFile previewFile(QString(), "KExiv2ImagePreview");
-        previewFile.setAutoDelete(true);
+        QByteArray data;
+        QBuffer buffer(data);
+        buffer.open(IO_WriteOnly);
+
         // A little bit compressed preview jpeg image to limit IPTC size.
-        preview.save(previewFile.name(), "JPEG");
-
-        QFile file(previewFile.name());
-        if ( !file.open(IO_ReadOnly) )
-            return false;
-
+        preview.save(&buffer, "JPEG");
         qDebug("JPEG image preview size: (%i x %i) pixels - %i bytes",
-               preview.width(), preview.height(), (int)file.size());
-
-        QByteArray data(file.size());
-        QDataStream stream( &file );
-        stream.readRawBytes(data.data(), data.size());
-        file.close();
+               preview.width(), preview.height(), (int)data.size());
 
         Exiv2::DataValue val;
         val.read((Exiv2::byte *)data.data(), data.size());
@@ -1285,10 +1279,15 @@ QString KExiv2::getExifTagString(const char* exifTagName, bool escapeCR) const
         Exiv2::ExifData::iterator it = exifData.findKey(exifKey);
         if (it != exifData.end())
         {
+#if (EXIV2_TEST_VERSION(0,17,91))
+            // See B.K.O #184156 comment #13
+            std::string val  = it->print(&exifData);
+            QString tagValue = QString::fromLocal8Bit(val.c_str());
+#else
             std::ostringstream os;
             os << *it;
             QString tagValue = QString::fromLocal8Bit(os.str().c_str());
-
+#endif
             if (escapeCR)
                 tagValue.replace("\n", " ");
 
